package com.displaynone.acss.components.auth.models.user.repository

import android.util.Log
import com.displaynone.acss.components.auth.models.user.repository.dto.LoginDTO
import com.displaynone.acss.components.auth.models.user.repository.dto.LoginResponseDTO
import com.displaynone.acss.components.auth.models.user.repository.dto.UpdateUserDTO
import com.displaynone.acss.components.auth.models.user.repository.dto.UserDTO
import com.displaynone.acss.config.Constants.serverUrl
import com.displaynone.acss.config.Network
import io.ktor.client.call.body
import io.ktor.client.request.get
import io.ktor.client.request.headers
import io.ktor.client.request.patch
import io.ktor.client.request.post
import io.ktor.client.request.setBody
import io.ktor.http.ContentType
import io.ktor.http.HttpHeaders
import io.ktor.http.HttpStatusCode
import io.ktor.http.contentType
import io.ktor.http.encodeURLPath
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext

class UserRepository {
    suspend fun login(login: String, password: String): Result<LoginResponseDTO> =
        withContext(Dispatchers.IO) {
            runCatching {
                val result = Network.client.post("$serverUrl/api/auth/login") {
                    headers {
                        append(HttpHeaders.ContentType, ContentType.Application.Json.toString())
                    }
                    contentType(ContentType.Application.Json)
                    setBody(LoginDTO(login, password))
                }

                if (result.status != HttpStatusCode.OK) error(result.status)

                result.body()
            }
        }

    suspend fun openDoor(token: String, code: String): Result<Int> = withContext(Dispatchers.IO) {
        runCatching {
            val result = Network.client.post("$serverUrl/api/acs/open") {
                headers {
                    append(HttpHeaders.Authorization, "Bearer $token")
                }
                contentType(ContentType.Application.Json)
                setBody("""{ "code": $code }""")
            }

            if (result.status != HttpStatusCode.OK) error("Status ${result.status}: ${result.body<String>()}")

            result.status.value
        }
    }

    suspend fun getInfo(token: String): Result<UserDTO> = withContext(Dispatchers.IO) {
        runCatching {
            val result = Network.client.get("$serverUrl/api/users/me") {
                headers {
                    append(HttpHeaders.Authorization, "Bearer $token")
                }
            }
            if (result.status != HttpStatusCode.OK) error("Status ${result.status}: ${result.body<String>()}")
            result.body()
        }
    }

    suspend fun getInfoByLogin(token: String, login: String): Result<UserDTO> =
        withContext(Dispatchers.IO) {
            runCatching {
                val encodedLogin = login.encodeURLPath()
                val result = Network.client.get("$serverUrl/api/users/login/$encodedLogin") {
                    headers {
                        append(HttpHeaders.Authorization, "Bearer $token")
                    }
                    setBody("""{ "code": "$encodedLogin" }""")
                }
                if (result.status != HttpStatusCode.OK) error("Status ${result.status}: ${result.body<String>()}")
                result.body()
            }
        }

    suspend fun changeRightsByLogin(
        login: String,
        token: String,
        isACSBlocked: Boolean
    ): Result<Unit> = withContext(Dispatchers.IO) {
        runCatching {
            val encodedLogin = login.encodeURLPath()
            val result = Network.client.patch("$serverUrl/api/users/login/$encodedLogin") {
                headers {
                    append(HttpHeaders.Authorization, "Bearer $token")
                }
                contentType(ContentType.Application.Json)
                setBody(
                    UpdateUserDTO(
                        password = null,
                        name = null,
                        photo = null,
                        position = null,
                        isACSBlocked = isACSBlocked
                    )
                )
            }
            if (result.status != HttpStatusCode.OK) {
                Log.w("UserRepository", "Status: ${result.status}, Body: ${result.body<String>()}")
                error("Status ${result.status}: ${result.body<String>()}")
            }
            Unit
        }
    }

}